# 2019-06-30 Bahar LAB, University of Pittsburgh  bahar@pitt.edu
# Written by Ji Young Lee, and edited by James Michael Krieger
# Calculate high affinity residues from trajectories of 
# Druggability molecular dynamics simulations

namespace eval ::highAff:: {
  namespace export highAff

  ############ Default parameter values (change or provide) #################
  # 1. PDB file name and path
  variable struc  ../md/sim.pdb     
  # 2. DCD file name and path
  variable dcd_in ../md/sim.dcd
  # 3. protein chain ID in pdb
  variable CHAIN all
  # 4. Probe molecule ID in pdb
  variable PROBE all
  # 5. first residue number of protein for analysis
  variable RESIDFIRST first
  # 6. last residue number of protein for analysis
  variable RESIDLAST  last  
  # 7. cutoff for assigning as high affinity residue
  variable cutoff 500
  ############ Default parameter values (change or provide) #################
}

proc wsplit {string sep} {
    set first [string first $sep $string]
    if {$first == -1} {
        return [list $string]
    } else {
        set l [string length $sep]
        set left [string range $string 0 [expr {$first-1}]]
        set right [string range $string [expr {$first+$l}] end]
        return [concat [list $left] [wsplit $right $sep]]
    }
}

# Take parameter values from input arguments as far as possible
for {set index 0} {$index < $argc -1} {incr index} {
  if {$index eq  0} {set ::highAff::struc [wsplit [lindex $argv $index] ,]}
  if {$index eq  1} {set ::highAff::dcd_in [wsplit [lindex $argv $index] ,]}
  if {$index eq  2} {set ::highAff::CHAIN [wsplit [lindex $argv $index] ,]}
  if {$index eq  3} {set ::highAff::PROBE [wsplit [lindex $argv $index] ,]}
  if {$index eq  4} {set ::highAff::RESIDFIRST [wsplit [lindex $argv $index] ,]}
  if {$index eq  5} {set ::highAff::RESIDLAST [wsplit [lindex $argv $index] ,]}
  if {$index eq  6} {set ::highAff::cutoff [wsplit [lindex $argv $index] ,]}
}

if { $argc < 8 } {
  # Argument 1 is the command itself
  puts "$index input arguments were provided."
  puts "The remaining values will be taken from the script."
}

if { $argc eq 2 && [lindex $argv 0] eq "help" } {
  puts ""
  puts "highaffresid.tcl takes up to 7 arguments, which must each contain no spaces. "
  puts "If you would like to include multiple values just put commas in between"
  puts "and the program will split them automatically."
  puts ""
  puts "The arguments are the following in a fixed order:"
  puts ""
  puts "struc: path to PDB file(s) for the starting structure of druggability simulation(s)"
  puts "you can provide either one path alone or one path per DCD file"
  puts "default value: ../md/sim.pdb"
  puts ""
  puts "dcd_in: path to DCD file(s) for druggability simulation(s)"
  puts "any number of DCD files can be provided as long as you meet the rule above about PDB files"
  puts "default value: ../md/sim.dcd"
  puts ""
  puts "CHAIN: the chains to analyze"
  puts "by default, all chains are analyzed"
  puts ""
  puts "PROBE: the probes to analyze"
  puts "by default, all probes are analyzed"
  puts ""
  puts "RESIDFIRST: the first residue to include in the analysis for each chain"
  puts "by default, the first residue in each chain is taken"
  puts "If you provide just one value, it will be used for all chains"
  puts ""
  puts "RESIDLAST: the last residue to include in the analysis for each chain"
  puts "by default, the last residue in each chain is taken"
  puts "If you provide just one value, it will be used for all chains"
  puts ""
  puts "To set a region of interest, please provide the same list to RESIDFIRST and RESIDLAST"
  puts "Please note this only works if you run the script for each chain separately"
  puts ""
  puts "cutoff: the binding value cutoff for assigning high affinity residues"
  puts "default value is 500"
  puts ""
  exit
}

proc findHighAffResids { strucs dcd_list CHAINS PROBES RESIDFIRST RESIDLAST cutoff } {

  # start loop for chains and probes
  set chainNum 0
  foreach CHAIN $CHAINS {
    puts ""
    puts "starting loop for chain $CHAIN"

    # make a list of resids for the resids foreach loop and handle regions of interest
    if { $RESIDFIRST eq $RESIDLAST } {
      puts "A region of interest was selected by using the same RESIDFIRST and RESIDLAST"
      puts "The residues of interest are $RESIDFIRST (a total of [llength $RESIDFIRST] residues)"
      # This only works with running the tcl script on one chain at a time.
      set resids $RESIDFIRST
    } else {
      puts "This analysis will run from residue [lindex $RESIDFIRST $chainNum] to residue [lindex $RESIDLAST $chainNum]"
      set resids {}
      for {set rrr [lindex $RESIDFIRST $chainNum] } {$rrr <= [lindex $RESIDLAST $chainNum]  } {incr rrr} {
        lappend resids $rrr
      }
    }

    foreach PROBE $PROBES {

      set ofile  [open out-$CHAIN-$PROBE.dat w]
      puts ""
      puts "starting loop for probe $PROBE"
      set count 0
      set interval 1  

      # start loop for dcd
      set dcdNum 0
      foreach dcd_in $dcd_list {
      
        if { [llength $strucs] > 1 } {
          set struc [lindex $strucs $chainNum]
        } else {
          set struc $strucs
        }
        mol load pdb $struc
        mol addfile $dcd_in type dcd first 0 last -1 step 1 waitfor -1

        set argNit [atomselect top "resname $PROBE and not hydrogen"]
        set first_frame 1
        set num_frames [molinfo top get numframes] 
        
        set highaffresids {} 
        # start loop for residue
        foreach rrr $resids {
          puts "starting loop for residue $rrr in chain $CHAIN with probe $PROBE"
          set DISTSUM 0
          # start loop for frame
          for {set f $first_frame} {$f < $num_frames} {incr f} {
            incr count
            set time [expr ($count-1)*$interval]
            molinfo top set frame $f
            set gluOxy [atomselect top "chain $CHAIN and protein and resid $rrr and not hydrogen"]
            $gluOxy frame $f
            $argNit frame $f
            set MECO  [measure contacts 4 $gluOxy $argNit]
            set NNNN  [llength [lindex $MECO 0]] 
            if {$NNNN > 0} {  
              for {set mmm 0} {$mmm < $NNNN } {incr mmm} {
                set KKKA  [lindex [lindex $MECO 0 ] $mmm ]
                set KKKB  [lindex [lindex $MECO 1 ] $mmm ]
                set DIST  [measure bond [list [list $KKKA] [list $KKKB]]]    
                set DISTSUM [expr $DISTSUM+ 1/(($DIST)*($DIST))]
              }
            }
            $gluOxy delete
          }
          # end loop for frame
          puts $ofile "$rrr  $DISTSUM"
          if { $DISTSUM > $cutoff } {
            lappend highaffresids $rrr
          } 
        }
        # end loop for residue
        animate delete all
        $argNit delete
        incr dcdNum
      }
      # end loop for dcd list
      flush $ofile
      close $ofile

      if {[llength $highaffresids] > 0} {
        set ofile2  [open highaffresid.dat a]
        puts $ofile2 "$PROBE $CHAIN $highaffresids"
        flush $ofile2
        close $ofile2
      }

      set ofile2  [open out-$CHAIN-$PROBE-highaffresid.dat a]
      puts $ofile2 "$highaffresids"
      flush $ofile2
      close $ofile2

      set pfile [open probe-list.dat a]
      puts $pfile $PROBE
      flush $pfile
      close $pfile
    }
    # end loop for probes
    incr chainNum
    set cfile [open chain-list.dat a]
    puts $cfile $CHAIN
    flush $cfile
    close $cfile
  }
  # end loop chains
}
# end proc

proc findChainIDs { struc dcd_in } {
  mol load pdb $struc
  mol addfile $dcd_in type dcd first 0 last -1 step 1 waitfor -1
  set sel [atomselect top all]
  set chains [lsort -ascii -unique [$sel get chain]] 
  set ::highAff::CHAIN [lrange $chains 0 1]
  $sel delete
  animate delete all
}


proc findProbeTypes { struc dcd_in } {
  mol load pdb $struc
  mol addfile $dcd_in type dcd first 0 last -1 step 1 waitfor -1
  set sel [atomselect top "chain P"]
  set ::highAff::PROBE [lsort -ascii -unique [$sel get resname]] 
  $sel delete
  animate delete all
}


proc findResidRange { struc dcd_in chains } {
  mol load pdb $struc
  mol addfile $dcd_in type dcd first 0 last -1 step 1 waitfor -1
  
  foreach chain $chains {
    set sel [atomselect top "chain $chain"]
    set resids [$sel get resid]

    if { [lindex $::highAff::RESIDFIRST 0] eq "first" } {
      lappend ::highAff::RESIDFIRST [lindex $resids 0]
    }

    if { [lindex $::highAff::RESIDLAST 0] eq "last" } {
      lappend ::highAff::RESIDLAST [lindex $resids end]
    }
  }

  if { [lindex $::highAff::RESIDFIRST 0] eq "first" } {
    set ::highAff::RESIDFIRST [lrange $::highAff::RESIDFIRST 1 end]
  }

  if { [lindex $::highAff::RESIDLAST 0] eq "last" } {
    set ::highAff::RESIDLAST [lrange $::highAff::RESIDLAST 1 end]
  }

  $sel delete
  animate delete all
}


if { $::highAff::CHAIN eq "all" } {
  findChainIDs $::highAff::struc $::highAff::dcd_in
}

if { $::highAff::PROBE eq "all" } {
  findProbeTypes $::highAff::struc $::highAff::dcd_in
}

if { $::highAff::RESIDFIRST eq "first" || $::highAff::RESIDLAST eq "last" } {
  findResidRange $::highAff::struc $::highAff::dcd_in $::highAff::CHAIN
}

set numChains [llength $::highAff::CHAIN]

if { [llength $::highAff::RESIDFIRST] eq 1 && $numChains > 1} {
  for {set index 1} {$index < $numChains} {incr index} {
    lappend ::highAff::RESIDFIRST [lindex $::highAff::RESIDFIRST 0]
  }
}

if { [llength $::highAff::RESIDLAST] eq 1 && $numChains > 1} {
  for {set index 1} {$index < $numChains} {incr index} {
    lappend ::highAff::RESIDLAST [lindex $::highAff::RESIDLAST 0]
  }
}

puts $::highAff::struc
puts $::highAff::dcd_in
puts $::highAff::CHAIN
puts $::highAff::PROBE
puts $::highAff::RESIDFIRST
puts $::highAff::RESIDLAST

findHighAffResids $::highAff::struc $::highAff::dcd_in $::highAff::CHAIN $::highAff::PROBE $::highAff::RESIDFIRST $::highAff::RESIDLAST $::highAff::cutoff

puts "High affinity residue analysis completed successfully"
exit
